@@ -23,30 +23,22 @@
 # terminate the script if any commands return a non-zero error code
 set -e
 
-cd source
-
-echo "************************************ Updating Core library"
-hg pull --cwd mupen64plus-core
-hg update --cwd mupen64plus-core $@
-
-echo "************************************ Updating console front-end"
-hg pull --cwd mupen64plus-ui-console
-hg update --cwd mupen64plus-ui-console $@
-
-echo "************************************ Updating SDL audio plugin"
-hg pull --cwd mupen64plus-audio-sdl
-hg update --cwd mupen64plus-audio-sdl $@
-
-echo "************************************ Updating SDL input plugin"
-hg pull --cwd mupen64plus-input-sdl
-hg update --cwd mupen64plus-input-sdl $@
-
-echo "************************************ Updating RSP HLE plugin"
-hg pull --cwd mupen64plus-rsp-hle
-hg update --cwd mupen64plus-rsp-hle $@
-
-echo "************************************ Updating Rice Video plugin"
-hg pull --cwd mupen64plus-video-rice
-hg update --cwd mupen64plus-video-rice $@
+if [ -z "$M64P_COMPONENTS" ]; then
+	M64P_COMPONENTS="core rom ui-console audio-sdl input-sdl rsp-hle video-rice video-glide64mk2"
+fi
 
+for component in ${M64P_COMPONENTS}; do
+	if [ "${component}" = "core" ]; then
+		component_type="library"
+	elif  [ "${component}" = "rom" ]; then
+		component_type=""
+	elif  [ "${component}" = "ui-console" ]; then
+		component_type="front-end"
+	else
+		component_type="plugin"
+	fi
 
+	echo "************************************ Update ${component} ${component_type}"
+	hg pull --cwd source/mupen64plus-${component}
+	hg update --cwd source/mupen64plus-${component} $@
+done
