@@ -319,10 +319,7 @@ EXPORT void * CALL DebugGetCPUDataPtr(m64p_dbg_cpu_data cpu_data_type)
     switch (cpu_data_type)
     {
         case M64P_CPU_PC:
-            if (r4300emu == CORE_PURE_INTERPRETER)
-                return &interp_addr;
-            else
-                return &PC->addr;
+            return &PC->addr;
         case M64P_CPU_REG_REG:
             return reg;
         case M64P_CPU_REG_HI:
