@@ -22,6 +22,31 @@
 
 #include <stdlib.h>
 #include <SDL.h>
+#if ! SDL_VERSION_ATLEAST(1,3,0)
+
+#define SDL_SCANCODE_ESCAPE SDLK_ESCAPE
+#define SDL_NUM_SCANCODES SDLK_LAST
+#define SDL_SCANCODE_F5 SDLK_F5
+#define SDL_SCANCODE_F7 SDLK_F7
+#define SDL_SCANCODE_F9 SDLK_F9
+#define SDL_SCANCODE_F10 SDLK_F10
+#define SDL_SCANCODE_F11 SDLK_F11
+#define SDL_SCANCODE_F12 SDLK_F12
+#define SDL_SCANCODE_P SDLK_p
+#define SDL_SCANCODE_M SDLK_m
+#define SDL_SCANCODE_RIGHTBRACKET SDLK_RIGHTBRACKET
+#define SDL_SCANCODE_LEFTBRACKET SDLK_LEFTBRACKET
+#define SDL_SCANCODE_F SDLK_f
+#define SDL_SCANCODE_SLASH SDLK_SLASH
+#define SDL_SCANCODE_G SDLK_g
+#define SDL_SCANCODE_RETURN SDLK_RETURN
+#define SDL_SCANCODE_0 SDLK_0
+#define SDL_SCANCODE_9 SDLK_9
+
+#define SDL_SetEventFilter(func, data) SDL_SetEventFilter(func)
+#define event_sdl_filter(userdata, event) event_sdl_filter(const event)
+
+#endif
 
 #define M64P_CORE_PROTOTYPES 1
 #include "main.h"
@@ -32,6 +57,7 @@
 #include "api/m64p_config.h"
 #include "plugin/plugin.h"
 #include "r4300/interupt.h"
+#include "r4300/reset.h"
 
 /* version number for CoreEvents config section */
 #define CONFIG_PARAM_VERSION 1.00
@@ -90,7 +116,7 @@ static const int NumJoyCommands = sizeof(JoyCmdName) / sizeof(const char *);
 
 static int JoyCmdActive[16];  /* if extra joystick commands are added above, make sure there is enough room in this array */
 
-static int KbdGamesharkPressed = 0;
+static int GamesharkActive = 0;
 
 /*********************************************************************************************************
 * static functions for eventloop.c
@@ -207,7 +233,7 @@ static int MatchJoyCommand(const SDL_Event *event, eJoyCommand cmd)
 /*********************************************************************************************************
 * sdl event filter
 */
-static int event_sdl_filter(const SDL_Event *event)
+static int SDLCALL event_sdl_filter(void *userdata, SDL_Event *event)
 {
     int cmd, action;
 
@@ -219,12 +245,43 @@ static int event_sdl_filter(const SDL_Event *event)
             break;
 
         case SDL_KEYDOWN:
+#if SDL_VERSION_ATLEAST(1,3,0)
+            if (event->key.repeat)
+                return 0;
+
+            event_sdl_keydown(event->key.keysym.scancode, event->key.keysym.mod);
+#else
             event_sdl_keydown(event->key.keysym.sym, event->key.keysym.mod);
+#endif
             return 0;
         case SDL_KEYUP:
+#if SDL_VERSION_ATLEAST(1,3,0)
+            event_sdl_keyup(event->key.keysym.scancode, event->key.keysym.mod);
+#else
             event_sdl_keyup(event->key.keysym.sym, event->key.keysym.mod);
+#endif
             return 0;
 
+#if SDL_VERSION_ATLEAST(2,0,0)
+        case SDL_WINDOWEVENT:
+            switch (event->window.event) {
+                case SDL_WINDOWEVENT_RESIZED:
+                    // call the video plugin.  if the video plugin supports resizing, it will resize its viewport and call
+                    // VidExt_ResizeWindow to update the window manager handling our opengl output window
+                    gfx.resizeVideoOutput(event->window.data1, event->window.data2);
+                    return 0;  // consumed the event
+                    break;
+            }
+            break;
+#else
+        case SDL_VIDEORESIZE:
+            // call the video plugin.  if the video plugin supports resizing, it will resize its viewport and call
+            // VidExt_ResizeWindow to update the window manager handling our opengl output window
+            gfx.resizeVideoOutput(event->resize.w, event->resize.h);
+            return 0;  // consumed the event
+            break;
+#endif
+
         // if joystick action is detected, check if it's mapped to a special function
         case SDL_JOYAXISMOTION:
         case SDL_JOYBUTTONDOWN:
@@ -236,13 +293,13 @@ static int event_sdl_filter(const SDL_Event *event)
                 if (action == 1) /* command was just activated (button down, etc) */
                 {
                     if (cmd == joyFullscreen)
-                        changeWindow();
+                        gfx.changeWindow();
                     else if (cmd == joyStop)
                         main_stop();
                     else if (cmd == joyPause)
                         main_toggle_pause();
                     else if (cmd == joySave)
-                        main_state_save(0, NULL); /* save in mupen64plus format using current slot */
+                        main_state_save(1, NULL); /* save in mupen64plus format using current slot */
                     else if (cmd == joyLoad)
                         main_state_load(NULL); /* load using current slot */
                     else if (cmd == joyIncrement)
@@ -250,27 +307,22 @@ static int event_sdl_filter(const SDL_Event *event)
                     else if (cmd == joyScreenshot)
                         main_take_next_screenshot();
                     else if (cmd == joyMute)
-                    {
-                        volumeMute();
-                        main_draw_volume_osd();
-                    }
+                        main_volume_mute();
                     else if (cmd == joyDecrease)
-                    {
-                        volumeDown();
-                        main_draw_volume_osd();
-                    }
+                        main_volume_down();
                     else if (cmd == joyIncrease)
-                    {
-                        volumeUp();
-                        main_draw_volume_osd();
-                    }
+                        main_volume_up();
                     else if (cmd == joyForward)
                         main_set_fastforward(1);
+                    else if (cmd == joyGameshark)
+                        event_set_gameshark(1);
                 }
                 else if (action == -1) /* command was just de-activated (button up, etc) */
                 {
                     if (cmd == joyForward)
                         main_set_fastforward(0);
+                    else if (cmd == joyGameshark)
+                        event_set_gameshark(0);
                 }
             }
 
@@ -303,14 +355,20 @@ void event_initialize(void)
             int device = event_str[1] - '0';
             if (!SDL_WasInit(SDL_INIT_JOYSTICK))
                 SDL_InitSubSystem(SDL_INIT_JOYSTICK);
+#if SDL_VERSION_ATLEAST(2,0,0)
+            SDL_JoystickOpen(device);
+#else
             if (!SDL_JoystickOpened(device))
                 SDL_JoystickOpen(device);
+#endif
         }
     }
 
     /* set up SDL event filter and disable key repeat */
+#if !SDL_VERSION_ATLEAST(2,0,0)
     SDL_EnableKeyRepeat(0, 0);
-    SDL_SetEventFilter(event_sdl_filter);
+#endif
+    SDL_SetEventFilter(event_sdl_filter, NULL);
 }
 
 int event_set_core_defaults(void)
@@ -347,24 +405,24 @@ int event_set_core_defaults(void)
         bSaveConfig = 1;
     }
 
-    ConfigSetDefaultFloat(l_CoreEventsConfig, "Version", CONFIG_PARAM_VERSION,  "Mupen64Plus CoreEvents config parameter set version number.  Please don't change");
+    ConfigSetDefaultFloat(l_CoreEventsConfig, "Version", CONFIG_PARAM_VERSION,  "Mupen64Plus CoreEvents config parameter set version number.  Please don't change this version number.");
     /* Keyboard presses mapped to core functions */
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdStop, SDLK_ESCAPE,          "SDL keysym for stopping the emulator");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdFullscreen, SDLK_LAST,      "SDL keysym for switching between fullscreen/windowed modes");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSave, SDLK_F5,              "SDL keysym for saving the emulator state");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdLoad, SDLK_F7,              "SDL keysym for loading the emulator state");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdIncrement, 0,               "SDL keysym for advancing the save state slot");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdReset, SDLK_F9,             "SDL keysym for resetting the emulator");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSpeeddown, SDLK_F10,        "SDL keysym for slowing down the emulator");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSpeedup, SDLK_F11,          "SDL keysym for speeding up the emulator");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdScreenshot, SDLK_F12,       "SDL keysym for taking a screenshot");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdPause, SDLK_p,              "SDL keysym for pausing the emulator");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdMute, SDLK_m,               "SDL keysym for muting/unmuting the sound");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdIncrease, SDLK_RIGHTBRACKET,"SDL keysym for increasing the volume");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdDecrease, SDLK_LEFTBRACKET, "SDL keysym for decreasing the volume");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdForward, SDLK_f,            "SDL keysym for temporarily going really fast");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdAdvance, SDLK_SLASH,        "SDL keysym for advancing by one frame when paused");
-    ConfigSetDefaultInt(l_CoreEventsConfig, kbdGameshark, SDLK_g,          "SDL keysym for pressing the game shark button");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdStop, SDL_SCANCODE_ESCAPE,          "SDL keysym for stopping the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdFullscreen, SDL_NUM_SCANCODES,      "SDL keysym for switching between fullscreen/windowed modes");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSave, SDL_SCANCODE_F5,              "SDL keysym for saving the emulator state");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdLoad, SDL_SCANCODE_F7,              "SDL keysym for loading the emulator state");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdIncrement, 0,                       "SDL keysym for advancing the save state slot");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdReset, SDL_SCANCODE_F9,             "SDL keysym for resetting the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSpeeddown, SDL_SCANCODE_F10,        "SDL keysym for slowing down the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdSpeedup, SDL_SCANCODE_F11,          "SDL keysym for speeding up the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdScreenshot, SDL_SCANCODE_F12,       "SDL keysym for taking a screenshot");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdPause, SDL_SCANCODE_P,              "SDL keysym for pausing the emulator");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdMute, SDL_SCANCODE_M,               "SDL keysym for muting/unmuting the sound");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdIncrease, SDL_SCANCODE_RIGHTBRACKET,"SDL keysym for increasing the volume");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdDecrease, SDL_SCANCODE_LEFTBRACKET, "SDL keysym for decreasing the volume");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdForward, SDL_SCANCODE_F,            "SDL keysym for temporarily going really fast");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdAdvance, SDL_SCANCODE_SLASH,        "SDL keysym for advancing by one frame when paused");
+    ConfigSetDefaultInt(l_CoreEventsConfig, kbdGameshark, SDL_SCANCODE_G,          "SDL keysym for pressing the game shark button");
     /* Joystick events mapped to core functions */
     ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyStop], "",       "Joystick event string for stopping the emulator");
     ConfigSetDefaultString(l_CoreEventsConfig, JoyCmdName[joyFullscreen], "", "Joystick event string for switching between fullscreen/windowed modes");
@@ -392,19 +450,15 @@ int event_set_core_defaults(void)
 void event_sdl_keydown(int keysym, int keymod)
 {
     /* check for the only 2 hard-coded key commands: Alt-enter for fullscreen and 0-9 for save state slot */
-    if (keysym == SDLK_RETURN && keymod & (KMOD_LALT | KMOD_RALT))
-    {
-        changeWindow();
-    }
-    else if (keysym >= SDLK_0 && keysym <= SDLK_9)
-    {
-        main_state_set_slot(keysym - SDLK_0);
-    }
+    if (keysym == SDL_SCANCODE_RETURN && keymod & (KMOD_LALT | KMOD_RALT))
+        gfx.changeWindow();
+    else if (keysym >= SDL_SCANCODE_0 && keysym <= SDL_SCANCODE_9)
+        main_state_set_slot(keysym - SDL_SCANCODE_0);
     /* check all of the configurable commands */
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdStop))
         main_stop();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdFullscreen))
-        changeWindow();
+        gfx.changeWindow();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdSave))
         main_state_save(0, NULL); /* save in mupen64plus format using current slot */
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdLoad))
@@ -412,10 +466,7 @@ void event_sdl_keydown(int keysym, int keymod)
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdIncrement))
         main_state_inc_slot();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdReset))
-    {
-        add_interupt_event(HW2_INT, 0);  /* Hardware 2 Interrupt immediately */
-        add_interupt_event(NMI_INT, 50000000);  /* Non maskable Interrupt after 1/2 second */
-    }
+        reset_soft();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdSpeeddown))
         main_speeddown(5);
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdSpeedup))
@@ -425,36 +476,21 @@ void event_sdl_keydown(int keysym, int keymod)
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdPause))
         main_toggle_pause();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdMute))
-    {
-        volumeMute();
-        main_draw_volume_osd();
-    }
+        main_volume_mute();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdIncrease))
-    {
-        volumeUp();
-        main_draw_volume_osd();
-    }
+        main_volume_up();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdDecrease))
-    {
-        volumeDown();
-        main_draw_volume_osd();
-    }
+        main_volume_down();
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdForward))
-    {
         main_set_fastforward(1);
-    }
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdAdvance))
-    {
         main_advance_one();
-    }
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdGameshark))
-    {
-        KbdGamesharkPressed = 1;
-    }
+        event_set_gameshark(1);
     else
     {
         /* pass all other keypresses to the input plugin */
-        keyDown(keymod, keysym);
+        input.keyDown(keymod, keysym);
     }
 
 }
@@ -471,15 +507,27 @@ void event_sdl_keyup(int keysym, int keymod)
     }
     else if (keysym == ConfigGetParamInt(l_CoreEventsConfig, kbdGameshark))
     {
-        KbdGamesharkPressed = 0;
+        event_set_gameshark(0);
     }
-    else keyUp(keymod, keysym);
+    else input.keyUp(keymod, keysym);
 
 }
 
 int event_gameshark_active(void)
 {
-    return KbdGamesharkPressed || JoyCmdActive[joyGameshark];
+    return GamesharkActive;
 }
 
+void event_set_gameshark(int active)
+{
+    // if boolean value doesn't change then just return
+    if (!active == !GamesharkActive)
+        return;
+
+    // set the button state
+    GamesharkActive = (active ? 1 : 0);
+
+    // notify front-end application that gameshark button state has changed
+    StateChanged(M64CORE_INPUT_GAMESHARK, GamesharkActive);
+}
 
