@@ -34,29 +34,18 @@ m64p_error close_rom(void);
 extern unsigned char* rom;
 extern int rom_size;
 
-typedef struct _rom_header
-{
-   unsigned char init_PI_BSB_DOM1_LAT_REG;  /* 0x00 */
-   unsigned char init_PI_BSB_DOM1_PGS_REG;  /* 0x01 */
-   unsigned char init_PI_BSB_DOM1_PWD_REG;  /* 0x02 */
-   unsigned char init_PI_BSB_DOM1_PGS_REG2; /* 0x03 */
-   unsigned int ClockRate;                  /* 0x04 */
-   unsigned int PC;                         /* 0x08 */
-   unsigned int Release;                    /* 0x0C */
-   unsigned int CRC1;                       /* 0x10 */
-   unsigned int CRC2;                       /* 0x14 */
-   unsigned int Unknown[2];                 /* 0x18 */
-   unsigned char nom[20];                   /* 0x20 */
-   unsigned int unknown;                    /* 0x34 */
-   unsigned int Manufacturer_ID;            /* 0x38 */
-   unsigned short Cartridge_ID;             /* 0x3C - Game serial number  */
-   unsigned short Country_code;             /* 0x3E */
-   unsigned int Boot_Code[1008];            /* 0x40 */
-} rom_header;
-
 extern unsigned char isGoldeneyeRom;
 
-extern rom_header*       ROM_HEADER;
+typedef struct _rom_params
+{
+   m64p_system_type systemtype;
+   int vilimit;
+   int aidacrate;
+   char headername[21];  /* ROM Name as in the header, removing trailing whitespace */
+} rom_params;
+
+extern m64p_rom_header   ROM_HEADER;
+extern rom_params        ROM_PARAMS;
 extern m64p_rom_settings ROM_SETTINGS;
 
 /* Supported rom compressiontypes. */
@@ -134,17 +123,14 @@ typedef struct _romdatabase_search
 
 typedef struct
 {
-    char* comment;
+    int have_database;
     romdatabase_search* crc_lists[256];
     romdatabase_search* md5_lists[256];
     romdatabase_search* list;
 } _romdatabase;
 
-extern romdatabase_entry empty_entry;
-
 void romdatabase_open(void);
 void romdatabase_close(void);
-romdatabase_entry* ini_search_by_md5(md5_byte_t* md5);
 /* Should be used by current cheat system (isn't), when cheat system is
  * migrated to md5s, will be fully depreciated.
  */
