@@ -36,174 +36,65 @@
 #include "api/debugger.h"
 #include "main/main.h"
 #include "main/rom.h"
+#include "main/util.h"
 #include "plugin/plugin.h"
 
 static unsigned char eeprom[0x800];
 static unsigned char mempack[4][0x8000];
 
-//#define DEBUG_PIF
-#ifdef DEBUG_PIF
-void print_pif(void)
+static char *get_eeprom_path(void)
 {
-    int i;
-    for (i=0; i<(64/8); i++)
-        DebugMessage(M64MSG_INFO, "%x %x %x %x | %x %x %x %x",
-                     PIF_RAMb[i*8+0], PIF_RAMb[i*8+1],PIF_RAMb[i*8+2], PIF_RAMb[i*8+3],
-                     PIF_RAMb[i*8+4], PIF_RAMb[i*8+5],PIF_RAMb[i*8+6], PIF_RAMb[i*8+7]);
-}
-#endif
-
-static unsigned char byte2bcd(int n)
-{
-    n %= 100;
-    return ((n / 10) << 4) | (n % 10);
+    return formatstr("%s%s.eep", get_savesrampath(), ROM_SETTINGS.goodname);
 }
 
-static void EepromCommand(unsigned char *Command)
+static void eeprom_format(void)
 {
-    time_t curtime_time;
-    struct tm curtime;
+    memset(eeprom, 0, sizeof(eeprom));
+}
 
-    switch (Command[2])
+static void eeprom_read_file(void)
+{
+    char *filename = get_eeprom_path();
+
+    switch (read_from_file(filename, eeprom, sizeof(eeprom)))
     {
-    case 0: // check
-#ifdef DEBUG_PIF
-        DebugMessage(M64MSG_INFO, "EepromCommand() check size");
-#endif
-        if (Command[1] != 3)
-        {
-            Command[1] |= 0x40;
-            if ((Command[1] & 3) > 0)
-                Command[3] = 0;
-            if ((Command[1] & 3) > 1)
-                Command[4] = (ROM_SETTINGS.savetype != EEPROM_16KB) ? 0x80 : 0xc0;
-            if ((Command[1] & 3) > 2)
-                Command[5] = 0;
-        }
-        else
-        {
-            Command[3] = 0;
-            Command[4] = (ROM_SETTINGS.savetype != EEPROM_16KB) ? 0x80 : 0xc0;
-            Command[5] = 0;
-        }
-        break;
-    case 4: // read
-    {
-        char *filename;
-        FILE *f;
-#ifdef DEBUG_PIF
-        DebugMessage(M64MSG_INFO, "EepromCommand() read 8-byte block %i", Command[3]);
-#endif
-        filename = (char *) malloc(strlen(get_savesrampath()) + strlen(ROM_SETTINGS.goodname)+4+1);
-        strcpy(filename, get_savesrampath());
-        strcat(filename, ROM_SETTINGS.goodname);
-        strcat(filename, ".eep");
-        f = fopen(filename, "rb");
-        if (f == NULL)
-        {
+        case file_open_error:
             DebugMessage(M64MSG_VERBOSE, "couldn't open eeprom file '%s' for reading", filename);
-            memset(eeprom, 0, 0x800);
-        }
-        else
-        {
-            if (fread(eeprom, 1, 0x800, f) != 0x800)
-                DebugMessage(M64MSG_WARNING, "fread() failed for 2kb eeprom file '%s'", filename);
-            fclose(f);
-        }
-        free(filename);
-        memcpy(&Command[4], eeprom + Command[3]*8, 8);
+            eeprom_format();
+            break;
+        case file_read_error:
+            DebugMessage(M64MSG_WARNING, "fread() failed for 2kb eeprom file '%s'", filename);
+            break;
+        default: break;
     }
-    break;
-    case 5: // write
+
+    free(filename);
+}
+
+static void eeprom_write_file(void)
+{
+    char *filename = get_eeprom_path();
+
+    switch (write_to_file(filename, eeprom, sizeof(eeprom)))
     {
-        char *filename;
-        FILE *f;
-#ifdef DEBUG_PIF
-        DebugMessage(M64MSG_INFO, "EepromCommand() write 8-byte block %i", Command[3]);
-#endif
-        filename = (char *) malloc(strlen(get_savesrampath()) + strlen(ROM_SETTINGS.goodname)+4+1);
-        strcpy(filename, get_savesrampath());
-        strcat(filename, ROM_SETTINGS.goodname);
-        strcat(filename, ".eep");
-        f = fopen(filename, "rb");
-        if (f == NULL)
-        {
-            DebugMessage(M64MSG_VERBOSE, "couldn't open eeprom file '%s' for reading", filename);
-            memset(eeprom, 0, 0x800);
-        }
-        else
-        {
-            if (fread(eeprom, 1, 0x800, f) != 0x800)
-                DebugMessage(M64MSG_WARNING, "fread() failed for 2kb eeprom file '%s'", filename);
-            fclose(f);
-        }
-        memcpy(eeprom + Command[3]*8, &Command[4], 8);
-        f = fopen(filename, "wb");
-        if (f == NULL)
-        {
+        case file_open_error:
             DebugMessage(M64MSG_WARNING, "couldn't open eeprom file '%s' for writing", filename);
-        }
-        else
-        {
-            if (fwrite(eeprom, 1, 0x800, f) != 0x800)
-                DebugMessage(M64MSG_WARNING, "fread() failed for 2kb eeprom file '%s'", filename);
-            fclose(f);
-        }
-        free(filename);
-    }
-    break;
-    case 6:
-#ifdef DEBUG_PIF
-        DebugMessage(M64MSG_INFO, "EepromCommand() RTC status query");
-#endif
-        // RTCstatus query
-        Command[3] = 0x00;
-        Command[4] = 0x10;
-        Command[5] = 0x00;
-        break;
-    case 7:
-#ifdef DEBUG_PIF
-        DebugMessage(M64MSG_INFO, "EepromCommand() read RTC block %i", Command[3]);
-#endif
-        // read RTC block
-        switch (Command[3]) {	// block number
-        case 0:
-            Command[4] = 0x00;
-            Command[5] = 0x02;
-            Command[12] = 0x00;
             break;
-        case 1:
-            DebugMessage(M64MSG_ERROR, "RTC command in EepromCommand(): read block %d", Command[2]);
+        case file_write_error:
+            DebugMessage(M64MSG_WARNING, "fwrite() failed for 2kb eeprom file '%s'", filename);
             break;
-        case 2:
-            time(&curtime_time);
-#if defined(WIN32)
-            localtime_s(&curtime, &curtime_time);
-#else
-            localtime_r(&curtime_time, &curtime);
-#endif
-            Command[4] = byte2bcd(curtime.tm_sec);
-            Command[5] = byte2bcd(curtime.tm_min);
-            Command[6] = 0x80 + byte2bcd(curtime.tm_hour);
-            Command[7] = byte2bcd(curtime.tm_mday);
-            Command[8] = byte2bcd(curtime.tm_wday);
-            Command[9] = byte2bcd(curtime.tm_mon + 1);
-            Command[10] = byte2bcd(curtime.tm_year);
-            Command[11] = byte2bcd(curtime.tm_year / 100);
-            Command[12] = 0x00;	// status
-            break;
-        }
-        break;
-    case 8:
-        // write RTC block
-        DebugMessage(M64MSG_ERROR, "RTC write in EepromCommand(): %d not yet implemented", Command[2]);
-        break;
-    default:
-        DebugMessage(M64MSG_ERROR, "unknown command in EepromCommand(): %x", Command[2]);
+        default: break;
     }
+
+    free(filename);
 }
 
-static void format_mempacks(void)
+static char *get_mempack_path(void)
+{
+    return formatstr("%s%s.mpk", get_savesrampath(), ROM_SETTINGS.goodname);
+}
+
+static void mempack_format(void)
 {
     unsigned char init[] =
     {
@@ -237,6 +128,155 @@ static void format_mempacks(void)
     }
 }
 
+static void mempack_read_file(void)
+{
+    char *filename = get_mempack_path();
+
+    switch (read_from_file(filename, mempack, sizeof(mempack)))
+    {
+        case file_open_error:
+            DebugMessage(M64MSG_VERBOSE, "couldn't open memory pack file '%s' for reading", filename);
+            mempack_format();
+            break;
+        case file_read_error:
+            DebugMessage(M64MSG_WARNING, "fread() failed for 128kb mempack file '%s'", filename);
+            break;
+        default: break;
+    }
+
+    free(filename);
+}
+
+static void mempack_write_file(void)
+{
+    char *filename = get_mempack_path();
+
+    switch (write_to_file(filename, mempack, sizeof(mempack)))
+    {
+        case file_open_error:
+            DebugMessage(M64MSG_WARNING, "couldn't open memory pack file '%s' for writing", filename);
+            break;
+        case file_write_error:
+            DebugMessage(M64MSG_WARNING, "fwrite() failed for 128kb mempack file '%s'", filename);
+            break;
+        default: break;
+    }
+
+    free(filename);
+}
+
+//#define DEBUG_PIF
+#ifdef DEBUG_PIF
+void print_pif(void)
+{
+    int i;
+    for (i=0; i<(64/8); i++)
+        DebugMessage(M64MSG_INFO, "%x %x %x %x | %x %x %x %x",
+                     PIF_RAMb[i*8+0], PIF_RAMb[i*8+1],PIF_RAMb[i*8+2], PIF_RAMb[i*8+3],
+                     PIF_RAMb[i*8+4], PIF_RAMb[i*8+5],PIF_RAMb[i*8+6], PIF_RAMb[i*8+7]);
+}
+#endif
+
+static unsigned char byte2bcd(int n)
+{
+    n %= 100;
+    return ((n / 10) << 4) | (n % 10);
+}
+
+static void EepromCommand(unsigned char *Command)
+{
+    time_t curtime_time;
+    struct tm *curtime;
+
+    switch (Command[2])
+    {
+    case 0: // check
+#ifdef DEBUG_PIF
+        DebugMessage(M64MSG_INFO, "EepromCommand() check size");
+#endif
+        if (Command[1] != 3)
+        {
+            Command[1] |= 0x40;
+            if ((Command[1] & 3) > 0)
+                Command[3] = 0;
+            if ((Command[1] & 3) > 1)
+                Command[4] = (ROM_SETTINGS.savetype != EEPROM_16KB) ? 0x80 : 0xc0;
+            if ((Command[1] & 3) > 2)
+                Command[5] = 0;
+        }
+        else
+        {
+            Command[3] = 0;
+            Command[4] = (ROM_SETTINGS.savetype != EEPROM_16KB) ? 0x80 : 0xc0;
+            Command[5] = 0;
+        }
+        break;
+    case 4: // read
+    {
+#ifdef DEBUG_PIF
+        DebugMessage(M64MSG_INFO, "EepromCommand() read 8-byte block %i", Command[3]);
+#endif
+        eeprom_read_file();
+        memcpy(&Command[4], eeprom + Command[3]*8, 8);
+    }
+    break;
+    case 5: // write
+    {
+#ifdef DEBUG_PIF
+        DebugMessage(M64MSG_INFO, "EepromCommand() write 8-byte block %i", Command[3]);
+#endif
+        eeprom_read_file();
+        memcpy(eeprom + Command[3]*8, &Command[4], 8);
+        eeprom_write_file();
+    }
+    break;
+    case 6:
+#ifdef DEBUG_PIF
+        DebugMessage(M64MSG_INFO, "EepromCommand() RTC status query");
+#endif
+        // RTCstatus query
+        Command[3] = 0x00;
+        Command[4] = 0x10;
+        Command[5] = 0x00;
+        break;
+    case 7:
+#ifdef DEBUG_PIF
+        DebugMessage(M64MSG_INFO, "EepromCommand() read RTC block %i", Command[3]);
+#endif
+        // read RTC block
+        switch (Command[3]) {	// block number
+        case 0:
+            Command[4] = 0x00;
+            Command[5] = 0x02;
+            Command[12] = 0x00;
+            break;
+        case 1:
+            DebugMessage(M64MSG_ERROR, "RTC command in EepromCommand(): read block %d", Command[2]);
+            break;
+        case 2:
+            time(&curtime_time);
+            curtime = localtime(&curtime_time);
+            Command[4] = byte2bcd(curtime->tm_sec);
+            Command[5] = byte2bcd(curtime->tm_min);
+            Command[6] = 0x80 + byte2bcd(curtime->tm_hour);
+            Command[7] = byte2bcd(curtime->tm_mday);
+            Command[8] = byte2bcd(curtime->tm_wday);
+            Command[9] = byte2bcd(curtime->tm_mon + 1);
+            Command[10] = byte2bcd(curtime->tm_year);
+            Command[11] = byte2bcd(curtime->tm_year / 100);
+            Command[12] = 0x00;	// status
+            break;
+        }
+        break;
+    case 8:
+        // write RTC block
+        DebugMessage(M64MSG_ERROR, "RTC write in EepromCommand(): %d not yet implemented", Command[2]);
+        break;
+    default:
+        DebugMessage(M64MSG_ERROR, "unknown command in EepromCommand(): %x", Command[2]);
+    }
+}
+
 static unsigned char mempack_crc(unsigned char *data)
 {
     int i;
@@ -266,7 +306,7 @@ static void internal_ReadController(int Control, unsigned char *Command)
         if (Controls[Control].Present)
         {
             BUTTONS Keys;
-            getKeys(Control, &Keys);
+            input.getKeys(Control, &Keys);
             *((unsigned int *)(Command + 3)) = Keys.Value;
 #ifdef COMPARE_CORE
             CoreCompareDataSync(4, Command+3);
@@ -280,7 +320,8 @@ static void internal_ReadController(int Control, unsigned char *Command)
         if (Controls[Control].Present)
         {
             if (Controls[Control].Plugin == PLUGIN_RAW)
-                if (controllerCommand) readController(Control, Command);
+                if (input.readController)
+                    input.readController(Control, Command);
         }
         break;
     case 3: // write controller pack
@@ -290,7 +331,8 @@ static void internal_ReadController(int Control, unsigned char *Command)
         if (Controls[Control].Present)
         {
             if (Controls[Control].Plugin == PLUGIN_RAW)
-                if (controllerCommand) readController(Control, Command);
+                if (input.readController)
+                    input.readController(Control, Command);
         }
         break;
     }
@@ -298,7 +340,6 @@ static void internal_ReadController(int Control, unsigned char *Command)
 
 static void internal_ControllerCommand(int Control, unsigned char *Command)
 {
-
     switch (Command[2])
     {
     case 0x00: // read status
@@ -356,31 +397,7 @@ static void internal_ControllerCommand(int Control, unsigned char *Command)
                     address &= 0xFFE0;
                     if (address <= 0x7FE0)
                     {
-                        char *filename;
-                        FILE *f;
-                        filename = (char *) malloc(strlen(get_savesrampath()) + strlen(ROM_SETTINGS.goodname)+4+1);
-                        strcpy(filename, get_savesrampath());
-                        strcat(filename, ROM_SETTINGS.goodname);
-                        strcat(filename, ".mpk");
-                        f = fopen(filename, "rb");
-                        if (f == NULL)
-                        {
-                            DebugMessage(M64MSG_VERBOSE, "couldn't open memory pack file '%s' for reading", filename);
-                            format_mempacks();
-                        }
-                        else
-                        {
-                            if (fread(mempack[0], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 1st 32kb mempack in file '%s'", filename);
-                            else if (fread(mempack[1], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 2nd 32kb mempack in file '%s'", filename);
-                            else if (fread(mempack[2], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 3rd 32kb mempack in file '%s'", filename);
-                            else if (fread(mempack[3], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 4th 32kb mempack in file '%s'", filename);
-                            fclose(f);
-                        }
-                        free(filename);
+                        mempack_read_file();
                         memcpy(&Command[5], &mempack[Control][address], 0x20);
                     }
                     else
@@ -395,7 +412,8 @@ static void internal_ControllerCommand(int Control, unsigned char *Command)
 #ifdef DEBUG_PIF
                 DebugMessage(M64MSG_INFO, "internal_ControllerCommand() Channel %i Command 2 controllerCommand (in Input plugin)", Control);
 #endif
-                if (controllerCommand) controllerCommand(Control, Command);
+                if (input.controllerCommand)
+                    input.controllerCommand(Control, Command);
                 break;
             default:
 #ifdef DEBUG_PIF
@@ -426,49 +444,9 @@ static void internal_ControllerCommand(int Control, unsigned char *Command)
                     address &= 0xFFE0;
                     if (address <= 0x7FE0)
                     {
-                        char *filename;
-                        FILE *f;
-                        filename = (char *) malloc(strlen(get_savesrampath()) + strlen(ROM_SETTINGS.goodname)+4+1);
-                        strcpy(filename, get_savesrampath());
-                        strcat(filename, ROM_SETTINGS.goodname);
-                        strcat(filename, ".mpk");
-                        f = fopen(filename, "rb");
-                        if (f == NULL)
-                        {
-                            DebugMessage(M64MSG_VERBOSE, "couldn't open memory pack file '%s' for reading", filename);
-                            format_mempacks();
-                        }
-                        else
-                        {
-                            if (fread(mempack[0], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 1st 32kb mempack in file '%s'", filename);
-                            else if (fread(mempack[1], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 2nd 32kb mempack in file '%s'", filename);
-                            else if (fread(mempack[2], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 3rd 32kb mempack in file '%s'", filename);
-                            else if (fread(mempack[3], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fread() failed for 4th 32kb mempack in file '%s'", filename);
-                            fclose(f);
-                        }
+                        mempack_read_file();
                         memcpy(&mempack[Control][address], &Command[5], 0x20);
-                        f = fopen(filename, "wb");
-                        if (f == NULL)
-                        {
-                            DebugMessage(M64MSG_WARNING, "couldn't open memory pack file '%s' for writing", filename);
-                        }
-                        else
-                        {
-                            if (fwrite(mempack[0], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fwrite() failed for 1st 32kb memory pack in file '%s'", filename);
-                            else if (fwrite(mempack[1], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fwrite() failed for 2nd 32kb memory pack in file '%s'", filename);
-                            else if (fwrite(mempack[2], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fwrite() failed for 3rd 32kb memory pack in file '%s'", filename);
-                            else if (fwrite(mempack[3], 1, 0x8000, f) != 0x8000)
-                                DebugMessage(M64MSG_WARNING, "fwrite() failed for 4th 32kb memory pack in file '%s'", filename);
-                            fclose(f);
-                        }
-                        free(filename);
+                        mempack_write_file();
                     }
                     Command[0x25] = mempack_crc(&Command[5]);
                 }
@@ -478,7 +456,8 @@ static void internal_ControllerCommand(int Control, unsigned char *Command)
 #ifdef DEBUG_PIF
                 DebugMessage(M64MSG_INFO, "internal_ControllerCommand() Channel %i Command 3 controllerCommand (in Input plugin)", Control);
 #endif
-                if (controllerCommand) controllerCommand(Control, Command);
+                if (input.controllerCommand)
+                    input.controllerCommand(Control, Command);
                 break;
             default:
 #ifdef DEBUG_PIF
@@ -549,7 +528,7 @@ void update_pif_write(void)
                 {
                     if (Controls[channel].Present &&
                             Controls[channel].RawData)
-                        controllerCommand(channel, &PIF_RAMb[i]);
+                        input.controllerCommand(channel, &PIF_RAMb[i]);
                     else
                         internal_ControllerCommand(channel, &PIF_RAMb[i]);
                 }
@@ -566,7 +545,7 @@ void update_pif_write(void)
         i++;
     }
     //PIF_RAMb[0x3F] = 0;
-    controllerCommand(-1, NULL);
+    input.controllerCommand(-1, NULL);
 }
 
 void update_pif_read(void)
@@ -596,7 +575,7 @@ void update_pif_read(void)
                 {
                     if (Controls[channel].Present &&
                             Controls[channel].RawData)
-                        readController(channel, &PIF_RAMb[i]);
+                        input.readController(channel, &PIF_RAMb[i]);
                     else
                         internal_ReadController(channel, &PIF_RAMb[i]);
                 }
@@ -608,6 +587,6 @@ void update_pif_read(void)
         }
         i++;
     }
-    readController(-1, NULL);
+    input.readController(-1, NULL);
 }
 
