@@ -24,6 +24,8 @@
 
 #include "api/m64p_plugin.h"
 
+extern m64p_error dummyvideo_PluginGetVersion(m64p_plugin_type *PluginType, int *PluginVersion,
+                                              int *APIVersion, const char **PluginNamePtr, int *Capabilities);
 extern void dummyvideo_ChangeWindow(void);
 extern int dummyvideo_InitiateGFX(GFX_INFO Gfx_Info);
 extern void dummyvideo_MoveScreen(int xpos, int ypos);
@@ -37,6 +39,7 @@ extern void dummyvideo_ViStatusChanged(void);
 extern void dummyvideo_ViWidthChanged(void);
 extern void dummyvideo_ReadScreen2(void *dest, int *width, int *height, int front);
 extern void dummyvideo_SetRenderingCallback(void (*callback)(int));
+extern void dummyvideo_ResizeVideoOutput(int width, int height);
 
 extern void dummyvideo_FBRead(unsigned int addr);
 extern void dummyvideo_FBWrite(unsigned int addr, unsigned int size);
