@@ -40,6 +40,8 @@
 #include "r4300.h"
 #include "macros.h"
 #include "exception.h"
+#include "reset.h"
+#include "new_dynarec/new_dynarec.h"
 
 #ifdef WITH_LIRC
 #include "main/lirc.h"
@@ -48,6 +50,9 @@
 unsigned int next_vi;
 int vi_field=0;
 static int vi_counter=0;
+
+int interupt_unsafe_state = 0;
+
 typedef struct _interupt_queue
 {
    int type;
@@ -318,38 +323,40 @@ void check_interupt(void)
 
 void gen_interupt(void)
 {
-
     if (stop == 1)
     {
         vi_counter = 0; // debug
         dyna_stop();
     }
-    if (savestates_job & LOADSTATE) 
+
+    if (!interupt_unsafe_state)
     {
-        savestates_load();
-        savestates_job &= ~LOADSTATE;
-        return;
+        if (savestates_get_job() == savestates_job_load)
+        {
+            savestates_load();
+            return;
+        }
+
+        if (reset_hard_job)
+        {
+            reset_hard();
+            reset_hard_job = 0;
+            return;
+        }
     }
    
     if (skip_jump)
     {
+        unsigned int dest = skip_jump;
+        skip_jump = 0;
+
         if (q->count > Count || (Count - q->count) < 0x80000000)
             next_interupt = q->count;
         else
             next_interupt = 0;
-        if (r4300emu == CORE_PURE_INTERPRETER)
-        {
-             interp_addr = skip_jump;
-             last_addr = interp_addr;
-        }
-        else
-        {
-            unsigned int dest = skip_jump;
-            skip_jump=0;
-            jump_to(dest);
-            last_addr = PC->addr;
-        }
-        skip_jump=0;
+        
+        last_addr = dest;
+        generic_jump_to(dest);
         return;
     } 
 
@@ -372,17 +379,18 @@ void gen_interupt(void)
             {
                 cheat_apply_cheats(ENTRY_VI);
             }
-            updateScreen();
+            gfx.updateScreen();
 #ifdef WITH_LIRC
             lircCheckInput();
 #endif
             SDL_PumpEvents();
+
             refresh_stat();
 
             // if paused, poll for input events
             if(rompause)
             {
-                osd_render();  // draw Paused message in case updateScreen didn't do it
+                osd_render();  // draw Paused message in case gfx.updateScreen didn't do it
                 VidExt_GL_SwapBuffers();
                 while(rompause)
                 {
@@ -493,13 +501,10 @@ void gen_interupt(void)
 
         case SP_INT:
             remove_interupt_event();
-            sp_register.sp_status_reg |= 0x303;
-            //sp_register.signal1 = 1;
-            sp_register.signal2 = 1;
-            sp_register.broke = 1;
-            sp_register.halt = 1;
+            sp_register.sp_status_reg |= 0x203;
+            // sp_register.sp_status_reg |= 0x303;
     
-            if (!sp_register.intr_break) return;
+            if (!(sp_register.sp_status_reg & 0x40)) return; // !intr_on_break
             MI_register.mi_intr_reg |= 0x01;
             if (MI_register.mi_intr_reg & MI_register.mi_intr_mask_reg)
                 Cause = (Cause | 0x400) & 0xFFFFFF83;
@@ -547,33 +552,14 @@ void gen_interupt(void)
             init_interupt();
             // clear the audio status register so that subsequent write_ai() calls will work properly
             ai_register.ai_status = 0;
+            // set ErrorEPC with the last instruction address
+            ErrorEPC = PC->addr;
             // reset the r4300 internal state
-            if (r4300emu == CORE_PURE_INTERPRETER) /* pure interpreter only */
+            if (r4300emu != CORE_PURE_INTERPRETER)
             {
-                // set ErrorEPC with last instruction address and set next instruction address to reset vector
-                ErrorEPC = interp_addr;
-                interp_addr = 0xa4000040;
-                last_addr = interp_addr;
-            }
-            else  /* decode-cached interpreter or dynamic recompiler */
-            {
-                int i;
-                // clear all the compiled instruction blocks
-                for (i=0; i<0x100000; i++)
-                {
-                    if (blocks[i])
-                    {
-                        free_block(blocks[i]);
-                        free(blocks[i]);
-                        blocks[i] = NULL;
-                    }
-                }
-                // re-initialize
+                // clear all the compiled instruction blocks and re-initialize
+                free_blocks();
                 init_blocks();
-                // jump to the start
-                ErrorEPC = PC->addr;
-                jump_to(0xa4000040);
-                last_addr = PC->addr;
             }
             // adjust ErrorEPC if we were in a delay slot, and clear the delay_slot and dyna_interp flags
             if(delay_slot==1 || delay_slot==3)
@@ -582,28 +568,37 @@ void gen_interupt(void)
             }
             delay_slot = 0;
             dyna_interp = 0;
+            // set next instruction address to reset vector
+            last_addr = 0xa4000040;
+            generic_jump_to(0xa4000040);
             return;
 
         default:
+            DebugMessage(M64MSG_ERROR, "Unknown interrupt queue event type %.8X.", q->type);
             remove_interupt_event();
             break;
     }
 
+#ifdef NEW_DYNAREC
+    if (r4300emu == CORE_DYNAREC) {
+        EPC = pcaddr;
+        pcaddr = 0x80000180;
+        Status |= 2;
+        Cause &= 0x7FFFFFFF;
+        pending_exception=1;
+    } else {
+        exception_general();
+    }
+#else
     exception_general();
+#endif
 
-    if(savestates_job & SAVESTATE)
+    if (!interupt_unsafe_state)
     {
-        if(savestates_job & SAVEPJ64STATE)
-        {
-            if(savestates_save_pj64() != -1)
-            {
-            savestates_job &= ~(SAVESTATE+SAVEPJ64STATE);
-            }
-        }
-        else
+        if (savestates_get_job() == savestates_job_save)
         {
             savestates_save();
-            savestates_job &= ~SAVESTATE;
+            return;
         }
     }
 }
