@@ -31,7 +31,7 @@
 void genadd_d(void)
 {
 #ifdef INTERPRET_ADD_D
-    gencallinterp((unsigned int)ADD_D, 0);
+    gencallinterp((unsigned int)cached_interpreter_table.ADD_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -46,7 +46,7 @@ void genadd_d(void)
 void gensub_d(void)
 {
 #ifdef INTERPRET_SUB_D
-   gencallinterp((unsigned int)SUB_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.SUB_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -61,7 +61,7 @@ void gensub_d(void)
 void genmul_d(void)
 {
 #ifdef INTERPRET_MUL_D
-   gencallinterp((unsigned int)MUL_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.MUL_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -76,7 +76,7 @@ void genmul_d(void)
 void gendiv_d(void)
 {
 #ifdef INTERPRET_DIV_D
-   gencallinterp((unsigned int)DIV_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.DIV_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -91,7 +91,7 @@ void gendiv_d(void)
 void gensqrt_d(void)
 {
 #ifdef INTERPRET_SQRT_D
-   gencallinterp((unsigned int)SQRT_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.SQRT_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -105,7 +105,7 @@ void gensqrt_d(void)
 void genabs_d(void)
 {
 #ifdef INTERPRET_ABS_D
-   gencallinterp((unsigned int)ABS_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.ABS_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -119,7 +119,7 @@ void genabs_d(void)
 void genmov_d(void)
 {
 #ifdef INTERPRET_MOV_D
-   gencallinterp((unsigned int)MOV_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.MOV_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -134,7 +134,7 @@ void genmov_d(void)
 void genneg_d(void)
 {
 #ifdef INTERPRET_NEG_D
-   gencallinterp((unsigned int)NEG_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.NEG_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_double[dst->f.cf.fs]));
@@ -148,7 +148,7 @@ void genneg_d(void)
 void genround_l_d(void)
 {
 #ifdef INTERPRET_ROUND_L_D
-   gencallinterp((unsigned int)ROUND_L_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.ROUND_L_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&round_mode);
@@ -163,7 +163,7 @@ void genround_l_d(void)
 void gentrunc_l_d(void)
 {
 #ifdef INTERPRET_TRUNC_L_D
-   gencallinterp((unsigned int)TRUNC_L_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.TRUNC_L_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&trunc_mode);
@@ -178,7 +178,7 @@ void gentrunc_l_d(void)
 void genceil_l_d(void)
 {
 #ifdef INTERPRET_CEIL_L_D
-   gencallinterp((unsigned int)CEIL_L_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CEIL_L_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&ceil_mode);
@@ -193,7 +193,7 @@ void genceil_l_d(void)
 void genfloor_l_d(void)
 {
 #ifdef INTERPRET_FLOOR_L_D
-   gencallinterp((unsigned int)FLOOR_L_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.FLOOR_L_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&floor_mode);
@@ -208,7 +208,7 @@ void genfloor_l_d(void)
 void genround_w_d(void)
 {
 #ifdef INTERPRET_ROUND_W_D
-   gencallinterp((unsigned int)ROUND_W_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.ROUND_W_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&round_mode);
@@ -223,7 +223,7 @@ void genround_w_d(void)
 void gentrunc_w_d(void)
 {
 #ifdef INTERPRET_TRUNC_W_D
-   gencallinterp((unsigned int)TRUNC_W_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.TRUNC_W_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&trunc_mode);
@@ -238,7 +238,7 @@ void gentrunc_w_d(void)
 void genceil_w_d(void)
 {
 #ifdef INTERPRET_CEIL_W_D
-   gencallinterp((unsigned int)CEIL_W_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CEIL_W_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&ceil_mode);
@@ -253,7 +253,7 @@ void genceil_w_d(void)
 void genfloor_w_d(void)
 {
 #ifdef INTERPRET_FLOOR_W_D
-   gencallinterp((unsigned int)FLOOR_W_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.FLOOR_W_D, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&floor_mode);
@@ -268,7 +268,7 @@ void genfloor_w_d(void)
 void gencvt_s_d(void)
 {
 #ifdef INTERPRET_CVT_S_D
-   gencallinterp((unsigned int)CVT_S_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CVT_S_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.fs]));
@@ -281,7 +281,7 @@ void gencvt_s_d(void)
 void gencvt_w_d(void)
 {
 #ifdef INTERPRET_CVT_W_D
-   gencallinterp((unsigned int)CVT_W_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CVT_W_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.fs]));
@@ -294,7 +294,7 @@ void gencvt_w_d(void)
 void gencvt_l_d(void)
 {
 #ifdef INTERPRET_CVT_L_D
-   gencallinterp((unsigned int)CVT_L_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CVT_L_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.fs]));
@@ -307,7 +307,7 @@ void gencvt_l_d(void)
 void genc_f_d(void)
 {
 #ifdef INTERPRET_C_F_D
-   gencallinterp((unsigned int)C_F_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_F_D, 0);
 #else
    gencheck_cop1_unusable();
    and_m32_imm32((unsigned int*)&FCR31, ~0x800000);
@@ -317,7 +317,7 @@ void genc_f_d(void)
 void genc_un_d(void)
 {
 #ifdef INTERPRET_C_UN_D
-   gencallinterp((unsigned int)C_UN_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_UN_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -336,7 +336,7 @@ void genc_un_d(void)
 void genc_eq_d(void)
 {
 #ifdef INTERPRET_C_EQ_D
-   gencallinterp((unsigned int)C_EQ_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_EQ_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -355,7 +355,7 @@ void genc_eq_d(void)
 void genc_ueq_d(void)
 {
 #ifdef INTERPRET_C_UEQ_D
-   gencallinterp((unsigned int)C_UEQ_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_UEQ_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -375,7 +375,7 @@ void genc_ueq_d(void)
 void genc_olt_d(void)
 {
 #ifdef INTERPRET_C_OLT_D
-   gencallinterp((unsigned int)C_OLT_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_OLT_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -394,7 +394,7 @@ void genc_olt_d(void)
 void genc_ult_d(void)
 {
 #ifdef INTERPRET_C_ULT_D
-   gencallinterp((unsigned int)C_ULT_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_ULT_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -414,7 +414,7 @@ void genc_ult_d(void)
 void genc_ole_d(void)
 {
 #ifdef INTERPRET_C_OLE_D
-   gencallinterp((unsigned int)C_OLE_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_OLE_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -433,7 +433,7 @@ void genc_ole_d(void)
 void genc_ule_d(void)
 {
 #ifdef INTERPRET_C_ULE_D
-   gencallinterp((unsigned int)C_ULE_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_ULE_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -453,7 +453,7 @@ void genc_ule_d(void)
 void genc_sf_d(void)
 {
 #ifdef INTERPRET_C_SF_D
-   gencallinterp((unsigned int)C_SF_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_SF_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -469,7 +469,7 @@ void genc_sf_d(void)
 void genc_ngle_d(void)
 {
 #ifdef INTERPRET_C_NGLE_D
-   gencallinterp((unsigned int)C_NGLE_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGLE_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -488,7 +488,7 @@ void genc_ngle_d(void)
 void genc_seq_d(void)
 {
 #ifdef INTERPRET_C_SEQ_D
-   gencallinterp((unsigned int)C_SEQ_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_SEQ_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -507,7 +507,7 @@ void genc_seq_d(void)
 void genc_ngl_d(void)
 {
 #ifdef INTERPRET_C_NGL_D
-   gencallinterp((unsigned int)C_NGL_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGL_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -527,7 +527,7 @@ void genc_ngl_d(void)
 void genc_lt_d(void)
 {
 #ifdef INTERPRET_C_LT_D
-   gencallinterp((unsigned int)C_LT_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_LT_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -546,7 +546,7 @@ void genc_lt_d(void)
 void genc_nge_d(void)
 {
 #ifdef INTERPRET_C_NGE_D
-   gencallinterp((unsigned int)C_NGE_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGE_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -566,7 +566,7 @@ void genc_nge_d(void)
 void genc_le_d(void)
 {
 #ifdef INTERPRET_C_LE_D
-   gencallinterp((unsigned int)C_LE_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_LE_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
@@ -585,7 +585,7 @@ void genc_le_d(void)
 void genc_ngt_d(void)
 {
 #ifdef INTERPRET_C_NGT_D
-   gencallinterp((unsigned int)C_NGT_D, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGT_D, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int*)(&reg_cop1_double[dst->f.cf.ft]));
