@@ -32,7 +32,7 @@
 void genadd_s(void)
 {
 #ifdef INTERPRET_ADD_S
-    gencallinterp((unsigned int)ADD_S, 0);
+    gencallinterp((unsigned int)cached_interpreter_table.ADD_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -47,7 +47,7 @@ void genadd_s(void)
 void gensub_s(void)
 {
 #ifdef INTERPRET_SUB_S
-    gencallinterp((unsigned int)SUB_S, 0);
+    gencallinterp((unsigned int)cached_interpreter_table.SUB_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -62,7 +62,7 @@ void gensub_s(void)
 void genmul_s(void)
 {
 #ifdef INTERPRET_MUL_S
-    gencallinterp((unsigned int)MUL_S, 0);
+    gencallinterp((unsigned int)cached_interpreter_table.MUL_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -77,7 +77,7 @@ void genmul_s(void)
 void gendiv_s(void)
 {
 #ifdef INTERPRET_DIV_S
-    gencallinterp((unsigned int)DIV_S, 0);
+    gencallinterp((unsigned int)cached_interpreter_table.DIV_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -92,7 +92,7 @@ void gendiv_s(void)
 void gensqrt_s(void)
 {
 #ifdef INTERPRET_SQRT_S
-   gencallinterp((unsigned int)SQRT_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.SQRT_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -106,7 +106,7 @@ void gensqrt_s(void)
 void genabs_s(void)
 {
 #ifdef INTERPRET_ABS_S
-   gencallinterp((unsigned int)ABS_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.ABS_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -120,7 +120,7 @@ void genabs_s(void)
 void genmov_s(void)
 {
 #ifdef INTERPRET_MOV_S
-   gencallinterp((unsigned int)MOV_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.MOV_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -133,7 +133,7 @@ void genmov_s(void)
 void genneg_s(void)
 {
 #ifdef INTERPRET_NEG_S
-   gencallinterp((unsigned int)NEG_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.NEG_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -147,7 +147,7 @@ void genneg_s(void)
 void genround_l_s(void)
 {
 #ifdef INTERPRET_ROUND_L_S
-   gencallinterp((unsigned int)ROUND_L_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.ROUND_L_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&round_mode);
@@ -162,7 +162,7 @@ void genround_l_s(void)
 void gentrunc_l_s(void)
 {
 #ifdef INTERPRET_TRUNC_L_S
-   gencallinterp((unsigned int)TRUNC_L_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.TRUNC_L_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&trunc_mode);
@@ -177,7 +177,7 @@ void gentrunc_l_s(void)
 void genceil_l_s(void)
 {
 #ifdef INTERPRET_CEIL_L_S
-   gencallinterp((unsigned int)CEIL_L_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CEIL_L_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&ceil_mode);
@@ -192,7 +192,7 @@ void genceil_l_s(void)
 void genfloor_l_s(void)
 {
 #ifdef INTERPRET_FLOOR_L_S
-   gencallinterp((unsigned int)FLOOR_L_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.FLOOR_L_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&floor_mode);
@@ -207,7 +207,7 @@ void genfloor_l_s(void)
 void genround_w_s(void)
 {
 #ifdef INTERPRET_ROUND_W_S
-   gencallinterp((unsigned int)ROUND_W_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.ROUND_W_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&round_mode);
@@ -222,7 +222,7 @@ void genround_w_s(void)
 void gentrunc_w_s(void)
 {
 #ifdef INTERPRET_TRUNC_W_S
-   gencallinterp((unsigned int)TRUNC_W_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.TRUNC_W_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&trunc_mode);
@@ -237,7 +237,7 @@ void gentrunc_w_s(void)
 void genceil_w_s(void)
 {
 #ifdef INTERPRET_CEIL_W_S
-   gencallinterp((unsigned int)CEIL_W_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CEIL_W_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&ceil_mode);
@@ -252,7 +252,7 @@ void genceil_w_s(void)
 void genfloor_w_s(void)
 {
 #ifdef INTERPRET_FLOOR_W_S
-   gencallinterp((unsigned int)FLOOR_W_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.FLOOR_W_S, 0);
 #else
    gencheck_cop1_unusable();
    fldcw_m16((unsigned short*)&floor_mode);
@@ -267,7 +267,7 @@ void genfloor_w_s(void)
 void gencvt_d_s(void)
 {
 #ifdef INTERPRET_CVT_D_S
-   gencallinterp((unsigned int)CVT_D_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CVT_D_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -280,7 +280,7 @@ void gencvt_d_s(void)
 void gencvt_w_s(void)
 {
 #ifdef INTERPRET_CVT_W_S
-   gencallinterp((unsigned int)CVT_W_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CVT_W_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -293,7 +293,7 @@ void gencvt_w_s(void)
 void gencvt_l_s(void)
 {
 #ifdef INTERPRET_CVT_L_S
-   gencallinterp((unsigned int)CVT_L_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.CVT_L_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.fs]));
@@ -306,7 +306,7 @@ void gencvt_l_s(void)
 void genc_f_s(void)
 {
 #ifdef INTERPRET_C_F_S
-   gencallinterp((unsigned int)C_F_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_F_S, 0);
 #else
    gencheck_cop1_unusable();
    and_m32_imm32((unsigned int*)&FCR31, ~0x800000);
@@ -316,7 +316,7 @@ void genc_f_s(void)
 void genc_un_s(void)
 {
 #ifdef INTERPRET_C_UN_S
-   gencallinterp((unsigned int)C_UN_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_UN_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -335,7 +335,7 @@ void genc_un_s(void)
 void genc_eq_s(void)
 {
 #ifdef INTERPRET_C_EQ_S
-   gencallinterp((unsigned int)C_EQ_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_EQ_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -354,7 +354,7 @@ void genc_eq_s(void)
 void genc_ueq_s(void)
 {
 #ifdef INTERPRET_C_UEQ_S
-   gencallinterp((unsigned int)C_UEQ_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_UEQ_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -374,7 +374,7 @@ void genc_ueq_s(void)
 void genc_olt_s(void)
 {
 #ifdef INTERPRET_C_OLT_S
-   gencallinterp((unsigned int)C_OLT_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_OLT_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -393,7 +393,7 @@ void genc_olt_s(void)
 void genc_ult_s(void)
 {
 #ifdef INTERPRET_C_ULT_S
-   gencallinterp((unsigned int)C_ULT_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_ULT_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -413,7 +413,7 @@ void genc_ult_s(void)
 void genc_ole_s(void)
 {
 #ifdef INTERPRET_C_OLE_S
-   gencallinterp((unsigned int)C_OLE_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_OLE_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -432,7 +432,7 @@ void genc_ole_s(void)
 void genc_ule_s(void)
 {
 #ifdef INTERPRET_C_ULE_S
-   gencallinterp((unsigned int)C_ULE_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_ULE_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -452,7 +452,7 @@ void genc_ule_s(void)
 void genc_sf_s(void)
 {
 #ifdef INTERPRET_C_SF_S
-   gencallinterp((unsigned int)C_SF_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_SF_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -468,7 +468,7 @@ void genc_sf_s(void)
 void genc_ngle_s(void)
 {
 #ifdef INTERPRET_C_NGLE_S
-   gencallinterp((unsigned int)C_NGLE_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGLE_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -487,7 +487,7 @@ void genc_ngle_s(void)
 void genc_seq_s(void)
 {
 #ifdef INTERPRET_C_SEQ_S
-   gencallinterp((unsigned int)C_SEQ_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_SEQ_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -506,7 +506,7 @@ void genc_seq_s(void)
 void genc_ngl_s(void)
 {
 #ifdef INTERPRET_C_NGL_S
-   gencallinterp((unsigned int)C_NGL_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGL_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -526,7 +526,7 @@ void genc_ngl_s(void)
 void genc_lt_s(void)
 {
 #ifdef INTERPRET_C_LT_S
-   gencallinterp((unsigned int)C_LT_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_LT_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -545,7 +545,7 @@ void genc_lt_s(void)
 void genc_nge_s(void)
 {
 #ifdef INTERPRET_C_NGE_S
-   gencallinterp((unsigned int)C_NGE_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGE_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -565,7 +565,7 @@ void genc_nge_s(void)
 void genc_le_s(void)
 {
 #ifdef INTERPRET_C_LE_S
-   gencallinterp((unsigned int)C_LE_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_LE_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
@@ -584,7 +584,7 @@ void genc_le_s(void)
 void genc_ngt_s(void)
 {
 #ifdef INTERPRET_C_NGT_S
-   gencallinterp((unsigned int)C_NGT_S, 0);
+   gencallinterp((unsigned int)cached_interpreter_table.C_NGT_S, 0);
 #else
    gencheck_cop1_unusable();
    mov_eax_memoffs32((unsigned int *)(&reg_cop1_simple[dst->f.cf.ft]));
