@@ -2,7 +2,7 @@
 #/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 # *   Mupen64plus - build_bundle_bin.sh                                     *
 # *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
-# *   Copyright (C) 2009 Richard Goedeken                                   *
+# *   Copyright (C) 2009-2013 Richard Goedeken                              *
 # *                                                                         *
 # *   This program is free software; you can redistribute it and/or modify  *
 # *   it under the terms of the GNU General Public License as published by  *
@@ -36,11 +36,13 @@ cd ${TEMPDIR}/source
 
 echo "************************************ Downloading Mupen64Plus module source code"
 hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-core
+hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-rom
 hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-ui-console
 hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-audio-sdl
 hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-input-sdl
 hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-rsp-hle
 hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-video-rice
+hg clone -r $1 http://bitbucket.org/richard42/mupen64plus-video-glide64mk2
 shift
 for dirname in ./mupen64plus-*; do rm -rf ${dirname}/.hg*; done
 
@@ -51,32 +53,13 @@ echo "************************************ Building Mupen64Plus modules"
 cd ..
 tar xzvf source/mupen64plus-core/tools/m64p_helper_scripts.tar.gz
 
-./m64p_build.sh $@
+./m64p_build.sh COREDIR=/usr/local/lib/ $@
 mv "test" "${OUTPUTDIR}"
 
 echo "************************************ Creating archive"
-mkdir "${OUTPUTDIR}/doc"
-
-cd source
-for dirname in mupen64plus-*; do
-  if [ -f ${dirname}/README ]; then
-    cp ${dirname}/README "../${OUTPUTDIR}/doc/README-${dirname}"
-  fi
-  if [ -f ${dirname}/LICENSES ]; then
-    cp ${dirname}/LICENSES "../${OUTPUTDIR}/doc/LICENSES-${dirname}"
-  fi
-  if [ -f ${dirname}/RELEASE ]; then
-    cp ${dirname}/RELEASE "../${OUTPUTDIR}/doc/RELEASE-${dirname}"
-  fi
-done
-find mupen64plus-core/doc/ -maxdepth 1 -type f |while read filename; do cp "$filename" "../${OUTPUTDIR}/doc/"; done
-cp mupen64plus-core/tools/install_binary_bundle.sh "../${OUTPUTDIR}/install.sh"
-cp mupen64plus-core/tools/uninstall_binary_bundle.sh "../${OUTPUTDIR}/uninstall.sh"
-cp mupen64plus-ui-console/doc/* "../${OUTPUTDIR}/"
-cd ..
+cp source/mupen64plus-core/tools/install_binary_bundle.sh "${OUTPUTDIR}/install.sh"
+cp source/mupen64plus-core/tools/uninstall_binary_bundle.sh "${OUTPUTDIR}/uninstall.sh"
 
 tar c "${OUTPUTDIR}" --owner 0 --group 0 --numeric-owner | gzip -n > "../${OUTPUTDIR}.tar.gz"
 cd ..
 rm -rf "${TEMPDIR}"
-
-
