@@ -2,7 +2,7 @@
 #/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 # *   Mupen64plus - build_modules_src.sh                                    *
 # *   Mupen64Plus homepage: http://code.google.com/p/mupen64plus/           *
-# *   Copyright (C) 2009 Richard Goedeken                                   *
+# *   Copyright (C) 2009-2013 Richard Goedeken                              *
 # *                                                                         *
 # *   This program is free software; you can redistribute it and/or modify  *
 # *   it under the terms of the GNU General Public License as published by  *
@@ -24,21 +24,21 @@
 set -e
 
 if [ $# -lt 2 ]; then
-    echo "Usage: build_modules_src.sh <tag-name> <build-name> [-norom]"
+    echo "Usage: build_modules_src.sh <tag-name> <build-name>"
     exit 1
 fi
 
-modules='mupen64plus-core mupen64plus-ui-console mupen64plus-audio-sdl mupen64plus-input-sdl mupen64plus-rsp-hle mupen64plus-video-rice'
+modules='mupen64plus-core mupen64plus-rom mupen64plus-ui-console mupen64plus-audio-sdl mupen64plus-input-sdl mupen64plus-rsp-hle mupen64plus-video-rice mupen64plus-video-glide64mk2'
 for modname in ${modules}; do
-  OUTPUTDIR="${modname}-$2"
   echo "************************************ Downloading and packaging module source code: ${modname}"
   rm -rf "tmp"
+  EXCLUDE="--exclude .hgtags --exclude .hg_archival.txt --exclude .hgignore"
+  TARTAG=""
+  OUTPUTDIR="${modname}-$2"
   hg clone --noupdate "http://bitbucket.org/richard42/$modname" "tmp"
-  EXCLUDE="--exclude tmp/.hgtags --exclude tmp/.hg_archival.txt tmp/.hgignore"
-  if [ $# -eq 3 -a "$3" = "-norom" -a "$modname" = "mupen64plus-core" ]; then
-    EXCLUDE="${EXCLUDE} --exclude tmp/roms"
-  fi
-  hg --repository tmp archive --no-decode --type tar --prefix "${OUTPUTDIR}/" ${EXCLUDE} -r $1 "${OUTPUTDIR}.tar"
-  gzip -n -f "${OUTPUTDIR}.tar"
+  cd tmp
+  hg archive --no-decode --type tar --prefix "${OUTPUTDIR}/" ${EXCLUDE} -r $1 "../${OUTPUTDIR}${TARTAG}.tar"
+  cd ..
+  gzip -n -f "${OUTPUTDIR}${TARTAG}.tar"
   rm -rf "tmp"
 done
